		#include <p16F877.inc>
		LIST		P=16F877
		__config	h'3F72'

		#include <mp3.inc>
;#define G10_STARTUP		; Unremark only if you have a G10, G12
#define F10_ONLY		; Unremark if using an F10
#define F10_OR_HIGHER		; Unremark if you are using an F10,G10,G12
;#define XTAL_14.725MHz		; Unremark if you are using 14.725MHz
#define XTAL_14.31818Hz		; Unremark if you are using 14.31818MHz
;#define XTAL_14.745MHz		; Unremark if you are using 14.745MHz

;#define VOLUME_PREAMP		; Unremark for volume control on preamp, leave remarked for
				; volume control on headphone output (NOT WORKING IN THIS REV, LEAVE UNDEFINED)

#define CONTINUOUS_PLAY		; Unremark if you want the next CD (directory) to play when current one finishes
;#define SIRCS15_12BIT		; Unremark for 15 bit Sony protocol, remark for 12 bit

;***************************************************************************
;*
;* Code organization:
;*
;* Bank 0 (0x0000 to 0x00FF) - Boot code
;* Bank 0 (0x0100 to 0x07FF) - Currently unused
;* Bank 1 (0x0800 to 0x0FFF) - Init code, tables for MP3-CD and HD, EEPROM control
;* Bank 2 (0x1000 to 0x17FF) - MP3-HDD code
;* Bank 3 (0x1800 to 0x1FFF) - MP3-CD code
;*
;* Update Firmware code is always 0x7B
;* MP3-HD code debug is always 0xAA
;***************************************************************************

			org 0
boot			clrf		PORTA		; clear A0-A5
			clrf		PCLATH
			goto		Start
			nop

int 			movwf           W_BKP		; store W
			movlw		0x10		; INT is ALWAYS at start of bank 2
	                movwf		PCLATH		; probably already set?
			goto		h'0000'

Start
                        ; F877 - initialise AD converter for digital IO

                        bsf             STATUS,RP0
                        movlw           b'00000110'
                        movwf           h'1f'           ; ADCON1
                       
			movlw		b'00110111'	; A5=input (MAS), A4=input(IR), A3=enable
			movwf		TRISA		; A2=buttons A1,A0=i2c
			movlw		b'10000000'	; B7 input, B0-6 outputs
			movwf		TRISB
			movlw		b'00000000'	; E all outputs for MUX
			movwf		TRISE
			movlw		b'11111111'	; C and D all inputs
			movwf		TRISC		; C0-C7 inputs (by default)
			movwf		TRISD		; D0-D7 inputs (by default)
			bcf		STATUS,RP0

			clrf		SHADOW_C


			; set up for reading PC
			call		SetIOLatchReadWriteBoot

			; test for magic signal
			bsf		PORTA,ENABLE
			movf		PORTD,0
			bcf		PORTA,ENABLE
			xorlw		d'123'
			btfsc		STATUS,Z
			goto		BootLoader

			; Start up MP3 code at 2K
			movlw 0x08
			movwf PCLATH
			goto h'0000'	;FIRMWARE_START

;***************************************************************************
;*
;* SetIOSRAMWrite
;*
;***************************************************************************
;*
;* Set PORTA and ports C/D for reading/writing to the PC latch/buffer
;*
;***************************************************************************/

SetIOLatchReadWriteBoot	movlw		DEMUX_LATCH_RDWR
			movwf		PORTE
			bsf		STATUS,RP0
			clrf		TRISC
			movlw		h'ff'
			movwf		TRISD
			bcf		STATUS,RP0
			return

;***************************************************************************
;*
;*
;*
;***************************************************************************
;*
;*
;*
;***************************************************************************/

BootLoader		; get command from PC
			call		SetIOLatchReadWriteBoot
			call		PCSwapBoot
			movf		PC_RECV_0,0		; get cmd
			addwf		PCL,1			; jump
			goto		cmd_nop_boot
			goto		cmd_read
			goto		cmd_write

;***************************************************************************
;*
;*
;*
;***************************************************************************
;*
;*
;*
;***************************************************************************/

cmd_nop_boot		movlw		h'55'
			movwf		PC_SEND
			goto		BootLoader

;***************************************************************************
;*
;*
;*
;***************************************************************************
;*
;*
;*
;***************************************************************************/

cmd_read                ; get address
			call		PCSwapBoot	; get address

			; select address
			bcf		STATUS,RP0	; bank 0
			movf		PC_RECV_0,0
			bsf		STATUS,RP1	; bank 2
			movwf		EEADRH
			bcf		STATUS,RP1	; bank 0
			movf		PC_RECV_1,0
			bsf		STATUS,RP1	; bank 2
			movwf		EEADR

			; do read
			bsf		STATUS,RP0	; bank 3
			bsf		EECON1,EEPGD
			bsf		EECON1,RD
			nop
			nop

			; get results
			bcf		STATUS,RP0	; bank 2
			movf		EEDATA,0
			bcf		STATUS,RP1	; bank 0
			movwf		TMP_LO
			bsf		STATUS,RP1	; bank 2
			movf		EEDATH,0
			bcf		STATUS,RP1	; bank 0
			movwf		TMP_HI

			; transmit to PC
			movf		TMP_HI,0
			movwf		PC_SEND
			call		PCSwapBoot
			movf		TMP_LO,0
			movwf		PC_SEND
			call		PCSwapBoot
			goto		BootLoader


;***************************************************************************
;*
;*
;*
;***************************************************************************
;*
;*
;*
;***************************************************************************/

cmd_write               ; get address
			call		PCSwapBoot		; get address

			; select address
			bcf		STATUS,RP0	; bank 0
			movf		PC_RECV_0,0
			btfsc		STATUS,Z	; if high part of address is 0 then PC is trying to make
			goto		SKIP_WRITE	; a write to protected address.  PIC cannot allow this!
			bsf		STATUS,RP1	; bank 2
			movwf		EEADRH
			bcf		STATUS,RP1	; bank 0
			movf		PC_RECV_1,0
			bsf		STATUS,RP1	; bank 2
			movwf		EEADR
			bcf		STATUS,RP1	; bank 0

			; get data
			call		PCSwapBoot

			; select data
			bcf		STATUS,RP0	; bank 0
			movf		PC_RECV_0,0
			bsf		STATUS,RP1	; bank 2
			movwf		EEDATH
			bcf		STATUS,RP1	; bank 0
			movf		PC_RECV_1,0
			bsf		STATUS,RP1	; bank 2
			movwf		EEDATA

			; do write
			bsf		STATUS,RP0	; bank 3
			bsf		EECON1,EEPGD
			bsf		EECON1,WREN
			bcf		INTCON,GIE
			movlw		h'55'
			movwf		EECON2
			movlw		h'AA'
			movwf		EECON2
			bsf		EECON1,WR
			nop
			nop
			bcf		EECON1,WREN

			bcf		STATUS,RP1	; bank 2
			bcf		STATUS,RP0	; bank 0

			; we're done
			goto		BootLoader

SKIP_WRITE
			call		PCSwapBoot	; PC expects this so do it but ignore results
			goto		BootLoader



;***************************************************************************
;*
;* PCSwap
;*
;***************************************************************************
;*
;* Receive 16-bits from the PC into PC_RECV_0 and PC_RECV_1. Send PC_SEND
;* (8-bits) back to the PC. Uses a simple parallel port based synchronous
;* protocol.
;* assumes we're in PC latch IO mode.
;*
;***************************************************************************/

PCSwapBoot
			; recover previous PORTC value
			movf		SHADOW_C,0
			movwf		PORTC
			bsf		PORTA,ENABLE

			; wait for PC signal
swaploopBoot		btfsc		PORTB,7			; wait for low
			goto		swaploopBoot

			movf		PORTD,0			; read/store data
			movwf		PC_RECV_0

			movf		PC_SEND,0
			andlw		b'00001111'		; echo low 4-bits
			iorlw		b'11100000'
			movwf		PORTC
			bsf		PORTC,4

swaploop2Boot		btfss		PORTB,7			; wait for high
			goto		swaploop2Boot

			movf		PORTD,0			; read/store data
			movwf		PC_RECV_1

			swapf		PC_SEND,0
			andlw		b'00001111'
			iorlw		b'11110000'
			movwf		PORTC
			bcf		PORTC,4
			bcf		PORTA,ENABLE

			movwf		SHADOW_C
			bcf		SHADOW_C,4

			return

;-----------------------------------------------------------------------------------------------
; keep boot code under 0x0100 bytes
;-----------------------------------------------------------------------------------------------

		#include <mp3_CD.asm>
		#include <mp3_INIT.asm>
		#include <mp3_HD.asm>

		END
